unit MySysUtils;

interface

uses
  Windows;

type
  PStrData = ^TStrData;
  TStrData = record
    Ident: Integer;
    Str: string;
  end;

  LongRec=packed record
    case integer of
      0:(Lo,Hi:Word);
      1:(Words:array[0..1] of Word);
      2:(Bytes:array[0..3] of Byte);
  end;

  IReadWriteSync = interface
    ['{7B108C52-1D8F-4CDB-9CDF-57E071193D3F}']
    procedure BeginRead;
    procedure EndRead;
    function BeginWrite: Boolean;
    procedure EndWrite;
  end;

  function FileExists(const FileName: string): Boolean;
  function DosToWin(s:string):string;
  function MyExtractFilePath(const FileName:string):string;
  procedure FreeAndNil(var Obj);
  function EnumStringModules(Instance: Longint; Data: Pointer): Boolean;
  function FindStringResource(Ident: Integer): string;
  function StrAlloc(Size: Cardinal): PChar;
  procedure StrDispose(Str: PChar);
  function StrCopy(Dest: PChar; const Source: PChar): PChar;

implementation

function FileAge(const FileName: string): Integer;
var
  Handle: THandle;
  FindData: TWin32FindData;
  LocalFileTime: TFileTime;
begin
  Handle := FindFirstFile(PChar(FileName), FindData);
  if Handle <> INVALID_HANDLE_VALUE then
  begin
    Windows.FindClose(Handle);
    if (FindData.dwFileAttributes and FILE_ATTRIBUTE_DIRECTORY) = 0 then
    begin
      FileTimeToLocalFileTime(FindData.ftLastWriteTime, LocalFileTime);
      if FileTimeToDosDateTime(LocalFileTime, LongRec(Result).Hi,
        LongRec(Result).Lo) then Exit;
    end;
  end;
  Result := -1;
end;

function FileExists(const FileName: string): Boolean;
begin
  Result := FileAge(FileName) <> -1;
end;

function DosToWin(s:string):string;
var
  ch:PChar;
begin
ch:=StrAlloc(Length(s)+1);
OemToAnsi(PChar(s),ch);
Result:=ch;
StrDispose(ch);
end;

function MyExtractFilePath(const FileName:string):string;
var
  i:integer;
begin
for i:=Length(FileName) downto 1 do
  if FileName[i]='\' then
    Break;
Result:=Copy(FileName,1,i);
end;

procedure FreeAndNil(var Obj);
var
  Temp: TObject;
begin
  Temp := TObject(Obj);
  Pointer(Obj) := nil;
  Temp.Free;
end;

function EnumStringModules(Instance: Longint; Data: Pointer): Boolean;
var
  Buffer: array [0..1023] of char;
begin
  with PStrData(Data)^ do
  begin
    SetString(Str, Buffer,
      LoadString(Instance, Ident, Buffer, sizeof(Buffer)));
    Result := Str = '';
  end;
end;

function FindStringResource(Ident: Integer): string;
var
  StrData: TStrData;
begin
  StrData.Ident := Ident;
  StrData.Str := '';
  EnumResourceModules(EnumStringModules, @StrData);
  Result := StrData.Str;
end;

function StrAlloc(Size:cardinal):PChar;
begin
Inc(Size,SizeOf(Cardinal));
GetMem(Result,Size);
Cardinal(Pointer(Result)^):=Size;
Inc(Result,SizeOf(Cardinal));
end;

procedure StrDispose(Str:PChar);
begin
if Str<>nil then
  begin
  Dec(Str,SizeOf(Cardinal));
  FreeMem(Str,Cardinal(Pointer(Str)^));
  end;
end;

function StrCopy(Dest: PChar; const Source: PChar): PChar;
asm
        PUSH    EDI
        PUSH    ESI
        MOV     ESI,EAX
        MOV     EDI,EDX
        MOV     ECX,0FFFFFFFFH
        XOR     AL,AL
        REPNE   SCASB
        NOT     ECX
        MOV     EDI,ESI
        MOV     ESI,EDX
        MOV     EDX,ECX
        MOV     EAX,EDI
        SHR     ECX,2
        REP     MOVSD
        MOV     ECX,EDX
        AND     ECX,3
        REP     MOVSB
        POP     ESI
        POP     EDI
end;

end.
 